@@ -15,7 +15,10 @@ pub fn binary_str_concat(xs: &CalcitItems) -> Result<Calcit, String> {
       s.push_str(&b.turn_string());
       Ok(Calcit::Str(s))
     }
-    (_, _) => Err(format!("expected 2 arguments, got: {}", primes::CrListWrap(xs.clone()))),
+    (_, _) => Err(format!(
+      "expected 2 arguments, got: {}",
+      primes::CrListWrap(xs.to_owned())
+    )),
   }
 }
 
@@ -31,7 +34,10 @@ pub fn trim(xs: &CalcitItems) -> Result<Calcit, String> {
       }
     }
     (Some(a), Some(b)) => Err(format!("trim expected 2 strings, but got: {} {}", a, b)),
-    (_, _) => Err(format!("expected 2 arguments, got: {}", primes::CrListWrap(xs.clone()))),
+    (_, _) => Err(format!(
+      "expected 2 arguments, got: {}",
+      primes::CrListWrap(xs.to_owned())
+    )),
   }
 }
 
@@ -47,9 +53,9 @@ pub fn turn_string(xs: &CalcitItems) -> Result<Calcit, String> {
   match xs.get(0) {
     Some(Calcit::Nil) => Ok(Calcit::Str(String::from(""))),
     Some(Calcit::Bool(b)) => Ok(Calcit::Str(b.to_string())),
-    Some(Calcit::Str(s)) => Ok(Calcit::Str(s.clone())),
-    Some(Calcit::Keyword(s)) => Ok(Calcit::Str(s.clone())),
-    Some(Calcit::Symbol(s, ..)) => Ok(Calcit::Str(s.clone())),
+    Some(Calcit::Str(s)) => Ok(Calcit::Str(s.to_owned())),
+    Some(Calcit::Keyword(s)) => Ok(Calcit::Str(s.to_owned())),
+    Some(Calcit::Symbol(s, ..)) => Ok(Calcit::Str(s.to_owned())),
     Some(Calcit::Number(n)) => Ok(Calcit::Str(n.to_string())),
     Some(a) => Err(format!("turn-string cannot turn this to string: {}", a)),
     None => Err(String::from("turn-string expected 1 argument, got nothing")),
@@ -90,7 +96,7 @@ pub fn replace(xs: &CalcitItems) -> Result<Calcit, String> {
     (Some(a), Some(b), Some(c)) => Err(format!("str:replace expected 3 strings, got: {} {} {}", a, b, c)),
     (_, _, _) => Err(format!(
       "str:replace expected 3 arguments, got: {}",
-      primes::CrListWrap(xs.clone())
+      primes::CrListWrap(xs.to_owned())
     )),
   }
 }
