@@ -28,7 +28,7 @@ pub fn load_core_snapshot() -> Result<snapshot::Snapshot, String> {
   let bytes = include_bytes!("./cirru/calcit-core.cirru");
   let core_content = String::from_utf8_lossy(bytes).to_string();
   let core_data = cirru_edn::parse(&core_content)?;
-  snapshot::load_snapshot_data(core_data)
+  snapshot::load_snapshot_data(core_data, "calcit-internal://calcit-core.cirru")
 }
 
 pub fn run_program(
@@ -103,6 +103,6 @@ pub fn load_module(path: &str, base_dir: &Path) -> Result<snapshot::Snapshot, St
   let content = fs::read_to_string(&fullpath).expect(&format!("expected Cirru snapshot {:?}", fullpath));
   let data = cirru_edn::parse(&content)?;
   // println!("reading: {}", content);
-  let snapshot = snapshot::load_snapshot_data(data)?;
+  let snapshot = snapshot::load_snapshot_data(data, &fullpath)?;
   Ok(snapshot)
 }
