@@ -1,5 +1,5 @@
 // CALCIT VERSION
-export const calcit_version = "0.4.24";
+export const calcit_version = "0.4.25";
 
 import { overwriteComparator, initTernaryTreeMap } from "@calcit/ternary-tree";
 import { parse } from "@cirru/parser.ts";
@@ -1207,6 +1207,8 @@ export let format_to_lisp = (x: CalcitValue): string => {
     }
     chunk += ")";
     return chunk;
+  } else if (typeof x === "string") {
+    return JSON.stringify("|" + x);
   } else {
     return x.toString();
   }
